@@ -6,7 +6,7 @@
     | modification, are permitted provided that the conditions mentioned |
     | in the accompanying LICENSE file are met.                          |
     +--------------------------------------------------------------------+
-    | Copyright (c) 2004-2013, Michael Wallner <mike@php.net>            |
+    | Copyright (c) 2004-2014, Michael Wallner <mike@php.net>            |
     +--------------------------------------------------------------------+
 */
 
@@ -176,9 +176,13 @@ static size_t php_http_curle_read_callback(void *data, size_t len, size_t n, voi
 {
 	php_http_message_body_t *body = ctx;
 
-	if (body) {
-		TSRMLS_FETCH_FROM_CTX(body->ts);
-		return php_stream_read(php_http_message_body_stream(body), data, len * n);
+	if (body && body->stream_id) {
+		php_stream *s = php_http_message_body_stream(body);
+
+		if (s) {
+			TSRMLS_FETCH_FROM_CTX(body->ts);
+			return php_stream_read(s, data, len * n);
+		} else abort();
 	}
 	return 0;
 }
@@ -1488,8 +1492,8 @@ static STATUS php_http_client_curl_handler_prepare(php_http_client_curl_handler_
 			}
 		}
 		php_http_buffer_dtor(&header);
-		curl_easy_setopt(curl->handle, CURLOPT_HTTPHEADER, curl->options.headers);
 	}
+	curl_easy_setopt(curl->handle, CURLOPT_HTTPHEADER, curl->options.headers);
 
 	/* attach request body */
 	if ((body_size = php_http_message_body_size(msg->body))) {
@@ -1504,6 +1508,11 @@ static STATUS php_http_client_curl_handler_prepare(php_http_client_curl_handler_
 		curl_easy_setopt(curl->handle, CURLOPT_READDATA, msg->body);
 		curl_easy_setopt(curl->handle, CURLOPT_INFILESIZE, body_size);
 		curl_easy_setopt(curl->handle, CURLOPT_POSTFIELDSIZE, body_size);
+	} else {
+		curl_easy_setopt(curl->handle, CURLOPT_IOCTLDATA, NULL);
+		curl_easy_setopt(curl->handle, CURLOPT_READDATA, NULL);
+		curl_easy_setopt(curl->handle, CURLOPT_INFILESIZE, 0L);
+		curl_easy_setopt(curl->handle, CURLOPT_POSTFIELDSIZE, 0L);
 	}
 
 	php_http_options_apply(&php_http_curle_options, enqueue->options, curl);
@@ -1798,7 +1807,7 @@ static STATUS php_http_client_curl_exec(php_http_client_t *h)
 				/* see http://msdn.microsoft.com/library/en-us/winsock/winsock/windows_sockets_error_codes_2.asp */
 				php_error_docref(NULL TSRMLS_CC, E_WARNING, "WinSock error: %d", WSAGetLastError());
 #else
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, strerror(errno));
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", strerror(errno));
 #endif
 				return FAILURE;
 			}
